@@ -1,5 +1,6 @@
 """
 =======
+
 PyScons
 =======
 
@@ -149,6 +150,15 @@ import hashlib
 import tempfile
 import base64
 from itertools import chain
+import os, errno
+
+def mkdir_p(path):
+    try:
+        os.makedirs(path)
+    except OSError as exc: # Python >2.5
+        if exc.errno == errno.EEXIST:
+            pass
+        else: raise
 
 class PyScanner(MF):
     """
@@ -172,7 +182,7 @@ class PyScanner(MF):
 
         imports = [m.__file__ for m in self.modules.values() if m.__file__ != None]
 
-        if str(node).split('.')[-1] in ["py"]:
+        if str(node).split('.')[-1] in ["py", "pyscons", "scons"]:
             contents = open(str(node)).read()
             D = re.findall("#"+"##(.+)", contents) + re.findall("#" +"DEPENDS(.+)", contents, re.I)
             D = [f for f in chain(*(f.split() for f in D))]
@@ -181,10 +191,6 @@ class PyScanner(MF):
                 f = os.path.join(d,f)
                 if os.path.exists(f): imports.append(f)
         
-        imports = [i for i in imports if "/site-packages/" not in i]
-        imports = [i for i in imports if "/SCons/" not in i]
-        imports = [i for i in imports if "/dist-packages/" not in i]
-
         imports = list(os.path.abspath(m) for m in imports if self.path_filter(m))
         return imports
 
@@ -193,8 +199,10 @@ def ToList(input):
     elif not Util.is_List(input): input = [input]
     return input
 
+_USED_DIRS = set()
+
 def _PyScript(env, target, source, command = "python $SOURCES", 
-                    capture=None, *args, **kwargs): 
+                    capture=None, execute=None, *args, **kwargs): 
     """
     A new Builder added to a scons environment which has the same syntax as Command.
     """
@@ -219,9 +227,13 @@ def _PyScript(env, target, source, command = "python $SOURCES",
     elif capture == None: pass
    
     command = [ env.subst(c, target=target, source=subst) for c in command ]
-    return env.Command(target, source, command, *args, **kwargs) 
+    command_sig = command
+
+    gen = lambda source, target, env, for_signature: command_sig if for_signature else command
+    return env.Command(target, source, CommandGeneratorAction(gen,{}), *args, **kwargs)
 
 def _PyScons(env, target, source, scons, execute=None, options="-Q", sig_options="", clean = True, tempdir = None):
+    global _USED_DIRS
     target = ToList(target) 
     source = ToList(source) 
 
@@ -235,25 +247,28 @@ def _PyScons(env, target, source, scons, execute=None, options="-Q", sig_options
         basename = os.path.join("temp", base64.b64encode(m.digest(), "+="), '')
     else: basename = tempdir
 
-    options += " temp=" + basename + " n=" + scons  
-    cmd = "scons %s %s -f%s %s" % (options, sig_options, scons, args)
-    cleancmd = cmd + " --clean"
+    USE_DIR = tuple(os.path.split(basename))
+    if USE_DIR in _USED_DIRS: raise ValueError("Cannot use temp directory %s in more than one PyScript or PyScons." % basename)
+    else: _USED_DIRS.add(USE_DIR)
 
-    if not os.path.exists(basename): env.Execute(Mkdir(basename))
+    vars = " temp=" + basename + " n=" + scons  
+
+    if not os.path.exists(basename): mkdir_p(basename)
     exe = os.path.join(basename, "EXECUTE")
     file = open(exe, 'w')
-    print >>file, "#!/bin/bash"
-    print >>file, cmd + " $*"
+    print >>file, "#!/bin/bash\nscons %s %s -f%s %s $*" % (vars, sig_options, scons, args)
     os.chmod(exe, 0755)
     file.close()
 
+    cmd = exe + " " + options
+    cleancmd = cmd + " --clean"
+
     if env.GetOption("clean") and not clean: 
         env.Execute(cleancmd)
         env.Execute(Delete(basename))
 
-    if not execute: cmd = [cmd]
+    if not execute: cmd = [cmd, cleancmd] if clean else [cmd]
     else: cmd = [execute + " " + basename]
-    if clean: cmd += [Delete(basename)]
 
     gen = lambda source, target, env, for_signature: sig if for_signature else cmd 
     return env.Command(target, source + [scons], CommandGeneratorAction(gen,{}))
@@ -264,12 +279,12 @@ PyScript = DefaultEnvironmentCall("PyScript")
 class PYTOOL:
     def __init__(self, path_filter = None, recursive=True):
         if path_filter == None: 
-            pythonos = os.__file__.strip("os.pyc").strip("os.py")
-            path_filter = lambda p: pythonos not in p
+            path_filter = lambda p: self.env.GetLaunchDir() == p[:len(self.env.GetLaunchDir())]
         self.path_filter = path_filter
         self.recursive = recursive
 
     def __call__(self, env):
+        self.env = env
         env.Append(SCANNERS = Scanner(function = PyScanner(self.path_filter), 
                     name = "PyScanner", skeys = ['.scons', '.pyscons', '.py', '.pyo', '.pyc'], recursive=self.recursive))
         env.AddMethod(_PyScript, "PyScript")
